//pressure
#include "BMP085_pressure.h"
#include <stdio.h>
#include <math.h>
#include "usb_cdc.h"

//=========Private Variables======
short ac1, ac2, ac3, b1, b2, b5, mb, mc, md;
unsigned short ac4, ac5, ac6;

void demo_pressure(void)
{
	BMP085_Calibration();
	while(1){
		printf("Temperature: %ld (in 0.1 deg C)", Temperature());
		printf(" Uncompensated = %ld\n",UncompensatedTemp());
		printf("Pressure: %ld Pa",Pressure());
		printf(" Uncompensated = %ld\n",UncompensatedPressure());
		printf("Altitude: %7.3fm\n",altitude());
		delay_ms(500);
		if(RxReady()){return;}
	}
}

void demo_pressureGuy(void)
{
	int v, n;

	BMP085_Calibration();
	while(1){
		v=0;
		for(n=0;n<100;n++){
			v = (Pressure()+v*4)/5;
			delay_ms(1);
		}
		printf("%ld\n",v/100);
		delay_ms(500);
	}
}

void BMP085_Calibration(void)
{
	ac1 = (i2c0_read(BMP085,0xaa)<<8|i2c0_read(BMP085,0xab));
	ac2 = (i2c0_read(BMP085,0xac)<<8|i2c0_read(BMP085,0xad));
	ac3 = (i2c0_read(BMP085,0xae)<<8|i2c0_read(BMP085,0xaf));
	ac4 = (i2c0_read(BMP085,0xb0)<<8|i2c0_read(BMP085,0xb1));
	ac5 = (i2c0_read(BMP085,0xb2)<<8|i2c0_read(BMP085,0xb3));
	ac6 = (i2c0_read(BMP085,0xb4)<<8|i2c0_read(BMP085,0xb5));
	b1  = (i2c0_read(BMP085,0xb6)<<8|i2c0_read(BMP085,0xb7));
	b2  = (i2c0_read(BMP085,0xb8)<<8|i2c0_read(BMP085,0xb9));
	mb  = (i2c0_read(BMP085,0xba)<<8|i2c0_read(BMP085,0xbb));
	mc  = (i2c0_read(BMP085,0xbc)<<8|i2c0_read(BMP085,0xbd));
	md  = (i2c0_read(BMP085,0xbe)<<8|i2c0_read(BMP085,0xbf));
	b5  = 0;
}

void BMP085_Print_Calibration(void)
{
	printf("\nCalibration Information:\n");
	printf("------------------------\n");
	printf("AC1 = %d\n", ac1);
	printf("\tAC2 = %d\n", ac2);
	printf("\tAC3 = %d\n", ac3);
	printf("\tAC4 = %d\n", ac4);
	printf("\tAC5 = %d\n", ac5);
	printf("\tAC6 = %d\n", ac6);
	printf("\tB1 = %d\n", b1);
	printf("\tB2 = %d\n", b2);
	printf("\tMB = %d\n", mb);
	printf("\tMC = %d\n", mc);
	printf("\tMD = %d\n", md);
	printf("\n");
}

unsigned int UncompensatedTemp(void)
{
	i2c0_write(BMP085,BMP085_Ctrl,BMP085_Temp);
	delay_ms(5);		// max time is 4.5ms
	return (i2c0_read(BMP085,BMP085_Rd_MSB)<<8|\
			i2c0_read(BMP085,BMP085_Rd_LSB));
}

unsigned int UncompensatedPressure(void)
{
	i2c0_write(BMP085,BMP085_Ctrl,BMP085_Pres|OSS<<6);
	delay_ms(26);		// max time is 25.5ms for ultra precision
	return ((i2c0_read(BMP085,BMP085_Rd_MSB)<<16|\
			 i2c0_read(BMP085,BMP085_Rd_LSB)<<8|\
			 i2c0_read(BMP085,BMP085_Rd_XLSB))>>(8-OSS));
}

int Temperature(void)
{
	int x1, x2;
	x1 = ((int)UncompensatedTemp() - ac6) * ac5 >> 15;
	x2 = ((int) mc << 11) / (x1 + md);
	b5 = x1 + x2;
	return ((b5 + 8) >> 4);		//temperature in 0.1 degrees C
}

int Pressure(void)
{
	int x1, x2;
	int b6, x3, b3, p;
	unsigned int b4, b7;

	UncompensatedPressure();
	Temperature();			//called to update global b5

	b6 = b5 - 4000;
	x1 = (b2 * ((b6 * b6) >> 12)) >> 11;
	x2 = (ac2 * b6) >> 11;
	x3 = x1 + x2;
	b3 = (((((int) ac1 * 4 + x3))<<OSS) + 2)>>2;
	x1 = (ac3 * b6) >> 13;
	x2 = (b1 * (b6 * b6 >> 12)) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (ac4 * (unsigned int) (x3 + 32768)) >> 15;
	b7 = (UncompensatedPressure() - b3) * (50000 >> OSS);
	p = b7 < 0x80000000 ? (b7 * 2) / b4 : (b7 / b4) * 2;
	x1 = (p >> 8) * (p >> 8);
	x1 = (x1 * 3038) >> 16;
	x2 = (-7357 * p) >> 16;
	return (p + ((x1 + x2 + 3791) >> 4));
}

float altitude(void)
{
	float temp;
	temp = (double) Pressure()/101325;
	temp = 1-pow(temp, 0.19029);
	return (44330*temp);
}

