#ifndef L3G4200D_gyroscope_H_
#define L3G4200D_gyroscope_H_
#include "lpc17xx.h"
#include "MMA7455_accelerometer.h"
#include <stdio.h>
#include "i2c.h"
#include "utility.h"
#include "usb_cdc.h"
#include "leds.h"
#include "PID.h"

float 	g_pitch_median,g_roll_median,g_yaw_median;
float 	g_pitch_previous,g_roll_previous,g_yaw_previous;
float 	g_pitch,g_roll,g_yaw;
float 	g_pitch_speed,g_roll_speed,g_yaw_speed;
float 	g_dt;
//char 	integrate;
int	gyro_saturation;

#define Khpf				0.985f
#define Klpf				(1.0-Khpf)

#define ResetControlLoop()\
	NVIC_DisableIRQ(TIMER3_IRQn);\
	NVIC_DisableIRQ(TIMER2_IRQn);\
	g_pitch = 0;\
	g_roll = 0;\
	g_yaw = 0;\
	g_filter_pitch = 0;\
	g_filter_roll = 0;\
	motors_off();

signed short Xgoffset,Ygoffset,Zgoffset;
float 	g_pitch,g_roll,g_yaw;
float 	g_filter_pitch,g_filter_roll;
float 	g_dt,g_dt_samples;
int		print_index;
char 	integrate;
float 	GyroSensitivity;

#define GyroSens250 	250/32768
#define GyroSens500 	500/32768
#define GyroSens2000 	2000/32768
#define L3G4200D		0xd0
#define WHO_AM_I		0x0F
#define OUT_X_L			0x28
#define OUT_X_H			0x29
#define OUT_Y_L			0x2A
#define OUT_Y_H			0x2B
#define OUT_Z_L			0x2C
#define OUT_Z_H			0x2D
#define CTRL_REG1		0x20		//The following indented bits can be
  #define PD				0x08	//OR'd into the CTRL_REG1 register
  #define XEN				0x01
  #define YEN				0x02
  #define ZEN				0x04
  #define DR400				0x80	//400Hz rate
  #define BW110				0x30	//110Hz cut off
#define CTRL_REG4 		0x23
  #define DPS250		0x00
  #define DPS500		0x10
  #define DPS2000		0x20
#define OUT_TEMP		0x26
#define ZYXOR			0x80		//Overrun
#define ZOR				0x40
#define YOR				0x20
#define XOR				0x10
#define ZYXDA			0x08		//Data available
#define ZDA				0x04
#define YDA				0x02
#define XDA				0x01

extern float g_X,g_Y,g_Z;
//Public
void demo_gyroscope(void);			//Print the pitch,roll and yaw angle in degrees
void demo_gyroscopeintegral(void);	//Print the pitch,roll and yaw angle in degrees
signed short PitchSpeed(void);		//pitch rate in degrees per second (dps)
signed short RollSpeed(void);		//roll rate in degrees per second (dps)
signed short YawSpeed(void);		//yaw rate in degrees per second (dps)
char Init_gyroscope(void);			//Initialise the sensor and calibrate
void print_Gyro(void);				//Print the roll,pitch and yaw in dps
void Calibrate_gyroscope(void);		//Calculate the offsets for the sensor
void IntegrateGyro(float dt, int min, int max);//Initialise the integration function
//Private
void init_timer3(float dt);			//timer 3 is used for the dt time
void TIMER3_IRQHandler(void);		//
#endif	//L3G4200D_gyroscope_H_
