#ifndef MMA7455_accelerometer_H_
#define MMA7455_accelerometer_H_
#include "lpc17xx.h"
#include <stdio.h>
#include "i2c.h"
#include "utility.h"
#include "math.h"
#include "usb_cdc.h"

#define Rad2Degrees	(float)(180/3.14159265358)

#define FullScale1G		63.0f		//64 counts per gravity
#define MMA7455			0x3a
#define XOUT8			0x06
#define YOUT8			0x07
#define ZOUT8			0x08
#define I2CAD			0x0D
#define XOUTL			0x00		//10 bit 2's compliment acceleration
#define XOUTH			0x01
#define YOUTL			0x02
#define YOUTH			0x03
#define ZOUTL			0x04
#define ZOUTH			0x05
#define XOFFL			0x10		//offset correction
#define XOFFH			0x11
#define YOFFL			0x12
#define YOFFH			0x13
#define ZOFFL			0x14
#define ZOFFH			0x15
#define MODE_CONTROL	0x16		//The following indented bits can be
  #define STANDBY			0x00	//OR'd into the MODE_CONTROL register
  #define MEASURE			0x01
  #define LEVEL				0x02
  #define PULSE				0x03
  #define LVL8G				0x00
  #define LVL4G				0x08
  #define LVL2G				0x04

volatile signed char Xoffset,Yoffset,Zoffset;
float RxPrevious,RyPrevious,RzPrevious;
extern float g_AngleX,g_AngleY,g_AngleZ;
//Public
void demo_accelerometer(void);		//Print the angles relative to earths gravity
//float Xaccelerometer(void);			//accelerometer readings scaled to degrees
//float Yaccelerometer(void);			//
//float Zaccelerometer(void);			//
char Init_accelerometer(void);		//Initialise the sensor and calibrate
void Calibrate_accelerometer(void);	//Calculate the offsets for the sensor
void print_Accelerometer(void);		//Print the raw readings (offset removed)
void UpdateGlobalAngles(void);		//Update the global XYZ angles relative to earths gravity
//Private
signed char Xmeasured(void);		//The raw readings with offset removed
signed char Ymeasured(void);		//
signed char Zmeasured(void);		//
#endif //MMA7455_accelerometer_H_
