#include "PID.h"
#include "battery.h"

//Start PID processing in the background using the timer3 callback.
//The function pointer can be used to change which PID function is run.
void Init_PID(void * fct)
{
	PID_callback = fct;							//Init callback
//Shrediquete Hover
	Kp = 0.0025;
	Ki = 0.0000015;
	Kd = 0.4;
//Shrediquete Acrobatic
	Kp = 0.45;
	Ki = 0.001;
	Kd = 0.0;
//Simple P term only
	Kp = 0.2;
	Ki = 0.0;
	Kd = 0.0;
}

//motor 0..100%
//throttle 0..100%
//stick_pitch -90 to +90 degrees
//g_pitch -90 to +90 degrees
float Roll_Error,Roll_Error_Integral=0,Roll_Error_Derivative,PID_roll;
void demo_testbed(void)
{
	g_dt = 0.005;								//sample rate in seconds
	printf("\nComplimentary_filter_demo.\n");
	printf("Khpf=%4.3f\n",Khpf);
	printf("tau=%4.3f\n",(Khpf*g_dt)/Klpf);
	printf("Sample_rate=%4.3f\n",g_dt);
	printf("Sensitivity=%4.0f dps\n",(float)GyroSensitivity*32768);
	printf("Kp=%4.3f,Ki=%4.3f,Kd=%4.3f\n\n",Kp,Ki,Kd);

	Init_PID(&TestBed);							//Set call back function
	init_timer3(g_dt);							//Enable timer3 integrals
	TraceResults(5);
	while(1){
		if(print_index ==1000){
			ResetControlLoop();
			PrintTraceResults();
			printf("Test complete\n");
			return;
		}
		if(RxReady()){
			switch(get_char()){
			case 'x':
			case 'X':
				ResetControlLoop();
				return;
			}
		}
	}
}

void TestBed(void)
{
	volatile float safe_value;
//	Roll_Error = roll_stick() - g_roll;			//degrees
	Roll_Error = 0-g_filter_roll;				//degrees
	CLAMP(Roll_Error, -40, +40);				//+-40 degrees
	Roll_Error_Integral += Roll_Error;			//degree
	CLAMP(Roll_Error_Integral, -10000, +10000);	//limit long term error
	Roll_Error_Derivative = (Roll_Error-g_Previous_Roll_Error)/g_dt;//degrees/sec
	g_Previous_Roll_Error = Roll_Error;
	PID_roll = Kp*Roll_Error\
			  +	Ki*Roll_Error_Integral\
			  + Kd*Roll_Error_Derivative;		//PID weightings

//	safe_value = throttle_stick() + PID_roll;
	safe_value = g_setpoint_thrust + PID_roll;	//set base throttle
	CLAMP(safe_value, 0, 40);					//safety precaution so
												//I only get 40% thrust
	motor_front(safe_value);
	g_PID_Roll_Error = Roll_Error;
	g_PID_Roll_Error_Integral = Roll_Error_Integral;
}

//This must be called every dt time intervals to function. This can be
//accomplished by placing it within the timer3 interrupt.
void PID(void)
{
	Update_PID_Error();							//Get the error
	Integrate_PID_Error();						//Integrate error
	Derivative_PID_Error();						//Differentiate error

	PsetRoll = g_RollError*Kp;
	PsetPitch = g_PitchError*Kp;
	PsetYaw = g_YawError*Kp;

	IsetRoll = g_roll_integral*Ki;
	IsetPitch = g_pitch_integral*Ki;
	IsetYaw = g_yaw_integral*Ki;

	DsetRoll = g_roll_derivative*Kd;
	DsetPitch = g_pitch_derivative*Kd;
	DsetYaw = g_yaw_derivative*Kd;
}

//Get the latest stick values and compare them to the
//sensor systems pitch, roll and yaw to get the relative
//error. The g_roll, g_pitch, g_yaw are measured in degrees while the
//control stick is measured in the range of +-50% which needs to be scaled
//to degrees. The Lf (Lift factor) constant performs the scaling
//function. For hover mode we allow a +-45degree result so Lf =4.5
void Update_PID_Error(void)
{
/*	g_RollError = g_roll - roll_stick();
	g_PitchError = g_pitch - pitch_stick();
	g_YawError = g_yaw - yaw_stick();*/
	g_RollError = g_roll - 0;
	g_PitchError = g_pitch - 0;
	g_YawError = g_yaw - 0;
}

//Integrate the error.
//To use this function it must be called at a regular g_dt interval.
//Typically this would be accomplished by calling this routine from
//your control loop (PID in this case) which is intern run every time
//the timer3 interrupt occurs.
void Integrate_PID_Error(void)
{
	g_roll_integral += g_RollError;
	g_pitch_integral += g_PitchError;
	g_yaw_integral += g_YawError;
}

//Derivative of the error
void Derivative_PID_Error(void)
{
	g_roll_derivative = g_RollError/g_dt;
	g_pitch_derivative = g_PitchError/g_dt;
	g_yaw_derivative = g_YawError/g_dt;
}

// Quad-Plus PID mixer
void Mixer(void)
{
	float roll,pitch,yaw;

	roll = PsetRoll + IsetRoll + DsetRoll;
	pitch = PsetPitch + IsetPitch + DsetPitch;
	yaw = PsetYaw + IsetYaw + DsetYaw;

	motor_left(throttle_stick()-roll-yaw);
	motor_right(throttle_stick()+roll-yaw);
	motor_front(throttle_stick()+pitch+yaw);
	motor_rear(throttle_stick()-pitch+yaw);
}
