#include "battery.h"
//Enable the ADC and set a callback function that will be called if the
//battery voltage is to low. The actual callback resides in the Systick
//interrupt and the battery level is checked every 1 second.
void Init_battery_alarm(void * fct)
{
	adcInit();
	battery_alarm_callback = fct;
}

//This is the function that is called when a low battery is detected.
void battery_alarm(void)
{
	if(battery_voltage1()<BatteryLowAlarm){
		//Do something
	}
}

/*****************************************************************************
** Function name:		ADCInit
**
** Descriptions:		initialize ADC channel
**
*****************************************************************************/
void adcInit(void)
{
// 	P0.23 is ADC0 - The PINSEL=01 (for ADC) with PINMODE 10 (no pullups/downs)
//	P0.24 is ADC1 - The PINSEL=01 (for ADC) with PINMODE 10 (no pullups/downs)
//	This sets various pins as analog inputs
//	Will need to change the clock divider if CPU clock is over 12MHz
//  The crystal is 12MHz which is multiplied up to CCLK=100MHz at startup.
//  This is then divided by 4 to give a PCLK=12MHz

	LPC_SC->PCONP |= (1 << 12); // Enable the ADC clock

	set_pin_function (0, 23, SECOND_FUNCTION );	//Select ADC0 pin function
	set_pin_function (0, 24, SECOND_FUNCTION );	//Select ADC1 pin function

	set_bits(&LPC_PINCON->PINMODE1, 14, 2, 2);	//Disable pullups on ADC0
	set_bits(&LPC_PINCON->PINMODE1, 16, 2, 2);	//Disable pullups on ADC1

	LPC_ADC->ADCR =
		( 1 << 0 ) |    	// SEL=1, select channel 0 on ADC0
//		( 2 << 8 ) |        // CLKDIV = 8.333Mhz
		( 50 << 8 ) |       // CLKDIV = 490khz
		( 0 << 16 ) | 		// BURST = 0, no BURST, software controlled
		( 1 << 21 ) |  		// PDN = 1, normal operation
		( 0 << 24 ) ;  		// START = 0 A/D conversion stopped

 }

/*****************************************************************************
** Function name:		adcRead
**
** Descriptions:
**
*****************************************************************************/
int adcRead(int channel)
{
	set_bits(&(LPC_ADC->ADCR), 0, 8, 1<<channel);	// set channel
	set_bits(&(LPC_ADC->ADCR), 24, 3, 1);  		    // start the conversion
	while(get_bits(LPC_ADC->ADGDR, 31, 1) == 0);    // wait until complete
	set_bits(&(LPC_ADC->ADCR), 23, 3, 0);	        // stop ADC (doesn't
													// appear to be needed)
	return get_bits(LPC_ADC->ADGDR, 4, 12);         // get result and return
}

float battery_voltage0(void)
{
	return((float)adcRead(0)*3.3f/4096);
}

//Vmeasured = Vbattery*(R9/(R9+R10))*mVperCount
// ADC value * max voltage * voltage divider / max value
float battery_voltage1(void)
{
	return ((float)adcRead(1) * 3.27 * ((r9+r10)/r9) / 4096 );
}

