#include "LPC17xx.h"
#include "utility.h"
#include "type.h"
#include "i2c.h"
#include "usb_cdc.h"
#include "MMA7455_accelerometer.h"
#include "L3G4200D_gyroscope.h"
#include "BMP085_pressure.h"
#include "leds.h"
#include "uart.h"
#include "radio.h"
#include "compass.h"
#include "motor.h"
#include "PID.h"
#include "battery.h"
#include <stdio.h>

// Resource Usage
// Systick -> 1ms event timer used by delay_ms()
// Timer0 -> 1us timer
// Timer3 -> integrator timing set by g_dt
// Timer2 -> sampled print statements with fixed timing

//What are the maximum sample rates for the sensors?
float 	g_AngleX,g_AngleY,g_AngleZ,Accel_High_Angle;
int 	BatteryTimer = 0;

// setup timer 0 to increment every microsecond
void init_timer0(void)
{
	LPC_TIM0->CTCR=0x00;     					// binary: 00000000
	LPC_TIM0->TC=0x00000000;     				// decimal 0
	LPC_TIM0->PR=0x00000018;     				// decimal 24
	LPC_TIM0->TCR=0x01;     					// binary: 00000001
}

//Interrupt handler for SysTick timer
void SysTick_Handler(void)
{
	void (*ptr)();
	current_time++;
	BatteryTimer++;
	if(BatteryTimer == 1000){					//run every second
		BatteryTimer = 0;						//reset BatteryTimer
		ptr = battery_alarm_callback;
		(*ptr)();								//call the battery alarm
	}
}

void DoNothing(void)
{
}

float OpenLoopMotor(float thrust, float thrust_increment, float target_angle)
{
	while(Reference_Angle()<target_angle){
		thrust=thrust+thrust_increment;
		CLAMP(thrust, 0, 40);			//safety precaution so
									//I only get 0 to 40% thrust
		motors_all(thrust);
		debug1("%4.3f, %4.3f\n",g_motor_front,Reference_Angle());
		delay_ms(50);				//let the arm stabilize
		debug1("%4.3f, %4.3f\n",g_motor_front,Reference_Angle());
	}
	return thrust;
}
/**************************************************************************/
int main(void)
{
	int i;
	static float f = 0;
	//The next two call back functions MUST be defined or your code will
	//hard fault because of bad pointers.
	Init_PID(&DoNothing);						//setup null call back
	Init_battery_alarm(&battery_alarm);			//set function to run

	Set_printf_IO(CDC,115200);					//Set to UART1,UART3 or CDC
	init_timer0();
	SysTick_Config (SystemCoreClock/1000);		//1kHz (1ms)
	I2C0Init( );								//I2c0 for sensors
	CDC_Init();
	leds_init();
	leds_all_on();
	Init_gyroscope();
	Init_accelerometer();
	adcInit();
	motors_off();								//turn off motors

	while(1){
		switch (get_char()){					//read default IO
		case 'q':
		case 'Q':
			leds_init();
			leds_all_off();
			printf("\nRed LED on.\n");
			led_on(LED_r);
			printf("\nWrite ESC0.\n");
			i2c0_write(IMU, ESC0H, 0x10);
			i2c0_write(IMU, ESC0L, 0x24);
			printf("\nRead ESC0 0x%x\n",i2c0_read(IMU, ESC0H));
			printf("\nRead ESC0 0x%x\n",i2c0_read(IMU, ESC0H));
			printf("\nRead ESC0 0x%x\n",i2c0_read(IMU, ESC0H));

			printf("\nWriting to RGB LED\n");
			i2c0_write(IMU, RED, 0xff);
			i2c0_write(IMU, GREEN, 0x05);
			i2c0_write(IMU, BLUE, 0x10);
			printf("\nRead Red IMU LED 0x%x\n",i2c0_read(IMU, RED));
			printf("\nRead Green IMU LED 0x%x\n",i2c0_read(IMU, GREEN));
			printf("\nRead Blue IMU LED 0x%x\n",i2c0_read(IMU, BLUE));

			putchar(get_char());
			led_off(LED_r);
			led_on(LED_g);
			break;

		case 'L':
		case 'l':
			IMU_LED_test();
			break;

		case 'E':
		case 'e':
			IMU_ESC_test();
			break;

		case 'f':
		case 'F':
			//Note for the CDC prints to work reliably the fastest you can
			//set the dt time is 4ms or 250Hz
			g_dt = 0.005;						//sample rate in seconds
			printf("\nComplimentary_filter_demo.\n");
			printf("Khpf=%4.3f\n",Khpf);
			printf("tau=%4.3f\n",(Khpf*g_dt)/Klpf);
			printf("Sample_rate=%4.3f\n",g_dt);
			init_timer3(g_dt);					//Enable timer3 integrals
			printf("Pitch,Roll,Yaw\n");
			while(1){
				printf("%4.3f, %4.3f, %4.3f\n",g_filter_pitch,g_filter_roll,g_yaw);
				delay_ms(50);
				if(RxReady()){
					NVIC_DisableIRQ(TIMER3_IRQn); //Disable integrator
					NVIC_DisableIRQ(TIMER2_IRQn); //Disable timed printouts
					break;
				}
			}
			break;
		case 'i':
		case 'I':
			debug1("Reference angle %4.3f\n",Reference_Angle());
			printf(",Thrust_vs_reference_angle\n");
				f=14.5;							//This is the minimum thrust
												//to start the motors

				while(f<29.0){					//speedy start until arm moves
					f=f+0.1;
					CLAMP(f, 0, 40);			//safety precaution so
												//I only get 0 to 40% thrust
					motors_all(f);
					delay_ms(50);				//let the arm stabilize
			    	debug1("%4.3f, %4.3f, MR1 = %d\n",g_motor_front,Reference_Angle(),LPC_PWM1->MR1);
				}
				f = OpenLoopMotor(f,0.1, -5.0);
				f = OpenLoopMotor(f,0.01, -3.0);
				g_setpoint_thrust = f;
				motors_all(g_setpoint_thrust);
				demo_testbed();
				ResetControlLoop();
			break;

		case 'n':
		case 'N':
			printf("Open loop thrust test.\n");
			printf("Thrust changes by 0.1 percent every 50ms.\n");
			printf(",Thrust_vs_roll_angle\n");
			init_timer3(0.010);					//Enable timer3 integrals
			delay_ms(1000);
			f=14.5;								//This is the minimum thrust
												//to start the motors
			f=25.0;
			motors_all(f);
			delay_ms(500);

			f = OpenLoopMotor(f,0.1, -15.0);
			f = OpenLoopMotor(f,0.001, -0.5);

			for(i=0;i<40;i++){
				motors_all(f);
				delay_ms(100);					//let the arm stabilize
				debug1("%4.3f, %4.3f\n",g_motor_front,g_filter_roll);
			}
			debug1("%4.3f, %4.3f, MR1 = %d\n",g_motor_front,Reference_Angle(),LPC_PWM1->MR1);
			for(i=0;i<40;i++){
				delay_ms(100);					//let the arm stabilize
				debug1("%4.3f, %4.3f\n",g_motor_front,g_filter_roll);
			}
			debug1("%4.3f, %4.3f, MR1 = %d\n",g_motor_front,Reference_Angle(),LPC_PWM1->MR1);

			ResetControlLoop();
			break;

		case '6':
			demo_radio();
			break;
		case '7':
			demo_testbed();
			break;
		case '9':
			demo_compass();
			break;
		case 'C':
		case 'c':
			g_ADC = 0;							//calibrate reference potentiometer
			for(i=0;i<20;i++){
				g_ADC += adcRead(0);
			};
			g_ADC = g_ADC/20;
			printf("Global ADC = %4.3f %d\n",(float)g_ADC,adcRead(0));
			Calibrate_accelerometer();
			Calibrate_gyroscope();
			BMP085_Calibration();

			printf("Reference angle %4.3f, ",Reference_Angle());
			printf("Roll angle%4.3f\n",g_filter_roll);

			printf("Move the arm to +30 degrees and press enter");
			get_char();
			for(i=0;i<20;i++){
				UpdateGlobalAngles();
				Accel_High_Angle += g_AngleY;
			}
			Accel_High_Angle = Accel_High_Angle/20;
			g_ADC_P30 = 0;							//calibrate reference potentiometer
			for(i=0;i<20;i++){
				g_ADC_P30 += adcRead(0);
			};
			g_ADC_P30 = g_ADC_P30/20;
	//		g_ADC_P30 = (g_ADC_P30-g_ADC)/Accel_High_Angle;
			g_ADC_P30 = (g_ADC_P30-g_ADC)/30.0;
			printf("Reference slope positive %4.3f angle %4.3f\n",g_ADC_P30,Accel_High_Angle);

			printf("Move the arm to -30 degrees and press enter");
			get_char();
			for(i=0;i<20;i++){
				UpdateGlobalAngles();
				Accel_High_Angle += g_AngleY;
			}
			Accel_High_Angle = Accel_High_Angle/20;
			g_ADC_M30 = 0;							//calibrate reference potentiometer
			for(i=0;i<20;i++){
				g_ADC_M30 += adcRead(0);
			};
			g_ADC_M30 = g_ADC_M30/20;
	//		g_ADC_M30 = (g_ADC_M30-g_ADC)/Accel_High_Angle;
			g_ADC_M30 = (g_ADC_M30-g_ADC)/-30.0;
			printf("Reference slope negative %4.3f angle %4.3f\n",g_ADC_M30,Accel_High_Angle);
			break;
		case 'R':
		case 'r':
			demo_radio();
			break;
		case '+':
			f += 1;
			printf("Thrust %4.3f\n",f);
			CLAMP(f, 0, 40);					//safety precaution so
												//I only get 0 to 40% thrust
			motors_all(f);
			break;
		case '-':
			f -= 1;
			printf("Thrust %4.3f\n",f);
			CLAMP(f, 0, 40);					//safety precaution so
												//I only get 0 to 40% thrust
			motors_all(f);
			break;
		case 'M':
		case 'm':
			demo_motor();
			break;
		default:
			motors_off();						//turn off motors
			printf("Demonstration programs\n");
			printf("Orient the sensors to a horizontal position and press 'c'");
			printf(" to calibrate the sensors.\n");
			printf("The calibration must be done prior to running any of the ");
			printf("other commands.\n\n");
			printf("c calibrate sensors and test bed\n");
			printf("f demonstrate the complimentary filter feeding an attitude display\n");
			printf("6 radio demo\n");
			printf("7 sensor filter results\n");
			printf("i PID demo work with motor set point\n");
			printf("t plot a thrust curve with MR print\n");
			printf("n plot a thrust curve without MR print\n");
			printf("q Test LEDs and ESC0\n");
			printf("r radio\n");
			printf("+ increase motor speed 10%\n");
			printf("- decrease motor speed 10%\n");
			printf("Core clock = %d Hz\n",SystemCoreClock);
			break;
		};
	}
}



