#include "uart.h"
#include "LPC17xx.h"
#include "leds.h"
#include "usb_cdc.h"
#include "utility.h"
#include "usb_cdc.h"

void demo_uart(void)
{
//	int c;
	UART1_Init(115200);								//Open all UART and CDC ports
	UART3_Init(115200);
	CDC_Init();
	while(1){
		ledg(0);
		ledr(0);
		PrintString1("Data sent to UART1\n");
		PrintString3("Data sent to UART3\n");
/*		putchar3('x');
		c = getchar3();
		led2(1);
		putchar1(c);
		USB_putchar(getchar1());
		led2(1);*/
		delay_ms(1000);
		if(RxReady()){
			return;
		}
	};
}

//UART1 is located on p0.15(tx) and p0.16(rx) and is connected to J6
void UART1_Init(int baudrate)
{
	int pclk;
	unsigned long int Fdiv;

	// UART pclk is 1/4 of SystemCoreClock
	pclk = SystemCoreClock/4;

	// Turn on power to UART
	LPC_SC->PCONP |=  PCUART1_POWERON;

	// Turn on peripheral clock
	LPC_SC->PCLKSEL0 &= ~(PCLK_UART1_MASK);
	LPC_SC->PCLKSEL0 |=  (0 << PCLK_UART1);		// PCLK_periph = CCLK/4

	// Set PINSEL so that P0.15 = TXD1, P0.16 = RXD1
	LPC_PINCON->PINSEL0 &= ~(1<<30 | 1<<31);	//clear bit 30 and 31
	LPC_PINCON->PINSEL0 |= 1<<30;				//01 for TxD1 p0.15
	LPC_PINCON->PINSEL1 &= ~0xfffC;				//clear bit 0 and 1
	LPC_PINCON->PINSEL1 |= 1;					//01 for RxD1 p0.16

	LPC_UART1->LCR = 0x83;		// 8 bits, no Parity, 1 Stop bit, DLAB=1
    Fdiv = (pclk/16)/baudrate;	// Set baud rate
    LPC_UART1->DLM = Fdiv / 256;
    LPC_UART1->DLL = Fdiv % 256;
    LPC_UART1->LCR = 0x03;		// 8 bits, no Parity, 1 Stop bit DLAB = 0
    LPC_UART1->FCR = 0x07;		// Enable and reset TX and RX FIFO
}

//UART3 is located on p0.0(tx) and p0.1(rx) and is connected to J5
void UART3_Init(int baudrate)
{
	int pclk;
	unsigned long int Fdiv;

	// UART pclk is 1/4 of SystemCoreClock
	pclk = SystemCoreClock/4;

	// Turn on power to UART
	LPC_SC->PCONP |= PCUART3_POWERON;

	// Turn on peripheral clock
	LPC_SC->PCLKSEL1 &= ~(PCLK_UART3_MASK);
	LPC_SC->PCLKSEL1 |=  (0 << PCLK_UART3);		// PCLK_periph = CCLK/4

	// Set PINSEL0 so that P0.0 = TXD3, P0.1 = RXD3
	LPC_PINCON->PINSEL0 &= ~0xfff0;
	LPC_PINCON->PINSEL0 |= ((1 << 3) | (1 << 1));

	LPC_UART3->LCR = 0x83;		// 8 bits, no Parity, 1 Stop bit, DLAB=1
	Fdiv = (pclk/16)/baudrate;	// Set baud rate
    LPC_UART3->DLM = Fdiv / 256;
    LPC_UART3->DLL = Fdiv % 256;
    LPC_UART3->LCR = 0x03;		// 8 bits, no Parity, 1 Stop bit DLAB = 0
    LPC_UART3->FCR = 0x07;		// Enable and reset TX and RX FIFO
}

void putchar1(unsigned char c)
{
	if(c == '\n'){
		while( (LPC_UART1->LSR & LSR_THRE) == 0 );	//Block until tx ready
		LPC_UART1->THR = '\r';
	}
	while( (LPC_UART1->LSR & LSR_THRE) == 0 );		//Block until tx ready
	LPC_UART1->THR = c;
}

// Function to send character over UART
void putchar3(unsigned char c)
{
	if(c == '\n'){
		while( (LPC_UART3->LSR & LSR_THRE) == 0 );	// Block until tx ready
		LPC_UART3->THR = '\r';
	}
	while( (LPC_UART3->LSR & LSR_THRE) == 0 );		// Block until tx ready
	LPC_UART3->THR = c;
}

unsigned char getchar1()
{
	while( (LPC_UART1->LSR & LSR_RDR) == 0 );  	// wait for Rx blocking
	return LPC_UART1->RBR; 						// read rx data
}

char rx_ready1()
{
	return (LPC_UART1->LSR & LSR_RDR); 			// result = 1 when ready
}

// Function to get character from UART
unsigned char getchar3()
{
	while( (LPC_UART3->LSR & LSR_RDR) == 0 );   // wait for Rx blocking
	return LPC_UART3->RBR; 						// read rx data
}

char rx_ready3()
{
	return (LPC_UART3->LSR & LSR_RDR); 			// result = 1 when ready
}

void PrintString1(char *pcString)
{
	int i = 0;
	while (pcString[i] != 0) {
		putchar1(pcString[i++]); 				// print each character
	}
}

// Function to prints the string out over the UART
void PrintString3(char *pcString)
{
	int i = 0;
	while (pcString[i] != 0) {
		putchar3(pcString[i++]); 				// print each character
	}
}
